 String.class_eval do
	#--
 	#	= String Class Methods for the Bash Shell
	#
	# * *Bash shell escaped characters:*
	#		- +! { } # & * ? [ ] ( ) = | ^ ; < > ` $ " ' \+
	# * *Bash shell special vars:*
	#		- +$$ $PPID $? $_ $! $PATH $IFS $HOME $UID $USER $# $@ $* $- $PWD+
	
	# Produces -> *String*
	#
	# Modifies a string to represent a file path at the Bash shell
	# command-line.
	#
	def bash_escape_file_path!
		raise ShellQuotes::SQError,
			'The NULL character is disallowed inside a Unix file path.'\
			if self =~ %r/\0/
		
		self.gsub!(%r{(!|#|&|\*|\?|\[|\]|\(|\)|\=|\||\^|;|<|>|`|\$|"|'|\\|\s)})\
			{ |match| '\\' + match } || self
	end
	
	# Similar to +bash_escape_file_path!+ but non-destructive.
	#
	def bash_escape_file_path
		clone = self.clone
		clone.bash_escape_file_path!
	end
	
	# Produces -> *String*
	#
	# Similar to +bash_escape_file_path+, but modifies a string which
	# represents an escaped file path into an unescaped string.
	#
	def bash_unescape_file_path!
		self.gsub!(
			%r{\\(!|#|&|\*|\?|\[|\]|\(|\)|\=|\||\^|;|<|>|`|\$|"|'|\\|\s)})\
			{ |match| match[-1] } || self
	end
	
	#	Similar to +bash_unescape_file_path!+ but non-destructive.
	#
	def bash_unescape_file_path
		clone = self.clone
		clone.bash_unescape_file_path!
	end
	
	# Produces -> *String*
	#
	# Modifies a string to represent a Bash shell-style escaped
	# command-line argument.
	#
	# This method does not take account of any shell variables. If these 
	# are included in the original string, they will be escaped, which is probably
	# not what you want.
	#
	# In case the original string is surrounded by backticks, single 
	# or double quotes then it will be passed on to
	# +bash_escape_substitute!+, +bash_escape_single_quoted!+, or 
	# +bash_escape_double_quoted!+ respectively.
	#
	def bash_escape_arg!
		unless self =~ %r{^(`|"|').*(`|"|')$}
			self.gsub!(
				%r{(!|\{|\}#|&|\*|\?|\[|\]|\(|\)|\=|\||\^|;|<|>|`|\$|"|'|\\)})\
				{ |match| '\\' + match } || self
		else
			self[0] == '`'\
				? self[1..-2].bash_escape_substitute!
				: self[0] == "'"\
					? self[1..-2].bash_escape_single_quoted!\
					: self[1..-2].bash_escape_double_quoted!
		end
	end
	
	#	Similar to +bash_escape_arg!!+ but non-destructive.
 	#
	def bash_escape_arg
		clone = self.clone
		clone.bash_escape_arg!
	end
	
	alias bash_single_quoted! bourne_single_quoted!
	
	alias bash_single_quoted bourne_single_quoted
	
	alias bash_double_quoted! bourne_double_quoted!
	
	alias bash_double_quoted bourne_double_quoted
	
	# Produces -> *String*
	#
	# Modifies a string to represent Bash shell-style command substitution.
	# Also takes care of nested command substitution by escaping backtick
	# characters where appropriate.
	#
	def bash_escape_substitute!
		(self.gsub!(%r/(`.+`)/) { |match| '$(' + match[1..-2] + ')' } || self)\
			.sub!(%r{.*}) { |match| "$(" + match + ")" }
	end
	
	#	Similar to +bash_escape_substitute!+ but non-destructive.
	#
	def bash_escape_substitute
		clone = self.clone
		clone.bash_escape_substitute!
	end
end
